require 'eventbrite_api_client/version'
require 'eventbrite_api_client/params'

require 'httparty'

class EventbriteApiClient
  include HTTParty
  base_uri 'https://www.eventbriteapi.com/v3'

  attr_reader :path

  def initialize(options)
    self.class.headers 'Authorization' => "Bearer #{options[:auth_token]}"

    @debug = options[:debug]
    @path = ""
  end

  def get(params = {})
    wrap_request(params) { |path, querystring| self.class.get(path, querystring) }
  end

  def post(params = {})
    wrap_request(params) { |path, querystring| self.class.post(path, querystring) }
  end

  def method_missing(method_sym, *args)
    them = self.dup

    them.path = "#{them.path}/#{method_sym.to_s}"
    them.path = "#{them.path}/#{args.first}" if args.first

    them
  end

  protected
    attr_writer :path

  private
    attr_reader :debug

    def ensure_json(response)
      if !is_json?(response)
        {
          'error' => 'NOT_FOUND',
          'error_description' => 'That is not a valid api endpoint',
          'status_code' => 404
        }
      else
        response
      end
    end

    def is_json?(response)
      response.respond_to?(:headers) && 
        response.headers['content-type'] == 'application/json'
    end

    def wrap_request(params, &block)
      _params = Params.new(params)

      puts "#{path}#{_params.format.to_s}" if debug

      ensure_json(block.call(path, query: _params.format))
    end
end
